using System;
using System.IO;

namespace Kosmaczewski.Oruga
{
	/// <summary>
	/// This class performs the transformation of the data provided by an IParser instance
	/// into the format handled by the ISerializer instance.
	/// </summary>
	public class Crisalida : IDisposable
	{
		private IParser parser;
		private ISerializer serializer;
		private Stream stream;

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="parser">The IParser instance with the data to be transformed.</param>
		/// <param name="serializer">The ISerializer instance to be used for output.</param>
		public Crisalida(IParser parser, ISerializer serializer)
		{
			this.parser = parser;
			this.serializer = serializer;
		}

		/// <summary>
		/// The Stream that will be used to write the output data.
		/// </summary>
		public Stream Stream
		{
			get
			{
				return this.stream;
			}

			set
			{
				this.stream = value;
			}
		}

		/// <summary>
		/// Performs the transformation and redirects the output to the specified file.
		/// </summary>
		/// <param name="filename">The complete path to the output file. 
		/// If the file does not exist, it will be created but if it exist it will be overwritten.</param>
		public void TransformToFile(string filename)
		{
			try
			{
				this.stream = new FileStream(filename, FileMode.Create);
				this.Transform();
			}
			catch (Exception e)
			{
				throw e;
			}
		}

		/// <summary>
		/// Performs the transformation and redirects the output to a generic stream.
		/// </summary>
		/// <param name="stream">The stream to write the data to.</param>
		public void TransformToStream(Stream stream)
		{
			this.stream = stream;
			this.Transform();
		}

		/// <summary>
		/// Performs the transformation of the data invoking the public methods
		/// of the parser and the serializer.
		/// </summary>
		private void Transform()
		{
			this.serializer.Source = this.parser.Parse();
			this.serializer.Write(this.stream);		
		}

		#region IDisposable Members

		/// <summary>
		/// Performs cleanup before garbage collection.
		/// </summary>
		public void Dispose()
		{
			this.stream.Close();
			this.stream = null;
		}

		#endregion
	}
}
