using System;
using System.Globalization;
using System.IO;
using System.Text;
using Kosmaczewski.Oruga.Rtf;
using Kosmaczewski.Oruga.XslFo;

namespace Kosmaczewski.Oruga.Rtf
{
	/// <summary>
	/// RtfParser encapsulates the RTF semantics for the tokens read by the
	/// RtfTextReader class. These semantics are used to build an XSL-FO compliant
	/// XML document that represents the contents of the RTF input stream.
	/// </summary>
	public class RtfParser : IParser
	{
		private RtfTextReader reader;
		private XslFoBuilder builder = new XslFoBuilder();
		private bool isValidRtf = false;
		private int currentDepth = 0;
		private string currentToken = "";
		private RtfTokenType currentTokenType;

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="filename">Path to the file to be parsed</param>
		public RtfParser(string filename)
		{
			if (File.Exists(filename))
			{
				this.reader = new RtfTextReader(new FileStream(filename, FileMode.Open));
			}
			else
			{
				throw new FileNotFoundException();
			}
		}

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="stream">A System.IO.Stream of RTF data.</param>
		public RtfParser(Stream stream)
		{
			this.reader = new RtfTextReader(stream);
		}

		/// <summary>
		/// This public method launches the parsing process and returns an instance of
		/// Kosmaczewski.Oruga.XslFo.XslFoDocument that represents the RTF data read.
		/// </summary>
		/// <returns>An XslFoDocument representing the data read.</returns>
		public XslFoDocument Parse()
		{
			ExecuteMainLoop();
			return this.builder.GetXslFoDocument();
		}

		/// <summary>
		/// Returns a string representation of the RTF data read, with a "BASIC-like" structure
		/// of the inner blocks.
		/// </summary>
		/// <returns>A string representing the RTF input data.</returns>
		public override string ToString()
		{
			return this.GetBlocksDisplay();
			//ExecuteMainLoop();
			//return this.builder.GetXslFoDocument().ToString();
		}

		#region Private Methods

		/// <summary>
		/// This method loops over the RtfTextReader and triggers the execution of private methods.
		/// </summary>
		private void ExecuteMainLoop()
		{
			while (this.reader.Read())
			{
				currentToken = this.reader.Token;
				currentDepth = this.reader.Depth;
				currentTokenType = this.reader.TokenType;
				switch (currentTokenType)
				{
					case RtfTokenType.Command:
						ExecuteCommand();
						break;

					case RtfTokenType.BlockCloser:
						break;

					case RtfTokenType.BlockOpener:
						break;

					case RtfTokenType.Text:
						AppendText();
						break;

					case RtfTokenType.WhiteSpace:
						AppendText();
						break;

					default:
						break;
				}
			}
		}

		/// <summary>
		/// This method is called when a command token is read by the RtfTextReader.
		/// </summary>
		private void ExecuteCommand()
		{
			switch (currentToken)
			{
				case @"\rtf1":
					if (!isValidRtf)
					{
						if (currentDepth == 1)
						{
							this.isValidRtf = true;
						}
					}
					break;

				case @"\pard":
					CreateElement(XslFoElementTypes.Paragraph);
					break;

				case @"\par":
					CloseBlock();
					break;

				case @"\plain":
					SetStyle(XslFoFontStyleTypes.Plain);
					break;

				case @"\b":
					SetStyle(XslFoFontStyleTypes.Bold);
					break;

				case @"\b0":
					goto case @"\b";

				case @"\i":
					SetStyle(XslFoFontStyleTypes.Italic);
					break;

				case @"\i0":
					goto case @"\i";

				case @"\ql":
					SetStyle(XslFoFontStyleTypes.LeftAlign);
					break;

				case @"\qc":
					SetStyle(XslFoFontStyleTypes.CenterAlign);
					break;

				case @"\qr":
					SetStyle(XslFoFontStyleTypes.RightAlign);
					break;

				default:
					break;
			}
		}

		/// <summary>
		/// This method is called whenever Escape or Text tokens are read.
		/// </summary>
		private void AppendText()
		{
			this.builder.AppendText(this.currentToken);
		}

		private void CloseBlock()
		{
			this.builder.CloseBlock();
		}

		private void CreateElement(XslFoElementTypes element)
		{
			this.builder.CreateElement(element);
		}

		private void SetStyle(XslFoFontStyleTypes style)
		{
			this.builder.SetStyle(style);
		}

		/// <summary>
		/// Creates the "blocks" or string representation of the RTF data.
		/// </summary>
		/// <returns>A string representing the RTF data</returns>
		private string GetBlocksDisplay()
		{
			StringBuilder sb = new StringBuilder();
			while (this.reader.Read())
			{
				currentToken = this.reader.Token;
				currentDepth = this.reader.Depth;
				currentTokenType = this.reader.TokenType;
				switch (currentTokenType)
				{
					case RtfTokenType.BlockCloser:
						for (int i = 0; i < currentDepth; i++)
						{
							sb.Append("\t");
						}
						sb.Append("End Block");
						sb.Append(Environment.NewLine);
						break;

					case RtfTokenType.BlockOpener:
						for (int i = 0; i < (currentDepth - 1); i++)
						{
							sb.Append("\t");
						}
						sb.Append("Begin Block");
						sb.Append(Environment.NewLine);
						break;

					case RtfTokenType.Command:
						for (int i = 0; i < currentDepth; i++)
						{
							sb.Append("\t");
						}
						sb.Append(currentToken);
						sb.Append(Environment.NewLine);
						break;

					case RtfTokenType.Text:
						for (int i = 0; i < currentDepth; i++)
						{
							sb.Append("\t");
						}
						sb.Append(currentToken);
						sb.Append(Environment.NewLine);
						break;

					case RtfTokenType.Escape:
						for (int i = 0; i < currentDepth; i++)
						{
							sb.Append("\t");
						}
						sb.Append(GetUnicodeChar());
						sb.Append(Environment.NewLine);
						break;

					default:
						break;
				}
			}
			return sb.ToString();		
		}

		/// <summary>
		/// Transforms the value of the Escape type tokens to a Unicode character.
		/// </summary>
		/// <returns></returns>
		private char GetUnicodeChar()
		{
			int code = int.Parse(currentToken.Substring(2, 2), NumberStyles.AllowHexSpecifier);
			return (char) code;
		}

		#endregion
	}
}
