using System;
using System.IO;
using System.Text;
using Kosmaczewski.Oruga;

namespace Kosmaczewski.Oruga.Rtf
{
	/// <summary>
	/// RtfTextReader provides forward-only, read-only access to a stream of RTF data (typically a file). 
	/// The current value refers to the token on which the reader is positioned. 
	/// The reader is advanced using the read method and the public properties reflect the 
	/// value and type of the current token.
	/// </summary>
	/// <remarks>This class is used exclusively by the RtfParser class and 
	/// it is not intended to be used by other clients. This class has been created following the 
	/// same structure and functionality as the System.Xml.XmlTextReader class.</remarks>
	internal sealed class RtfTextReader : IDisposable
	{
		private Stream stream;
		private int depth = 0;
		private RtfTokenType tokenType = RtfTokenType.Text;
		private StringBuilder token;
		private bool atEndOfStream = false;
		private char currentChar = (char)0;

		/// <summary>
		/// Public constructor.
		/// </summary>
		/// <param name="stream">A stream of RTF data (usually a System.IO.FileStream instance)</param>
		internal RtfTextReader(Stream stream)
		{
			this.stream = stream;
			ReadNextChar();
		}

		/// <summary>
		/// Indicates the nesting level of curly brackets "{".
		/// </summary>
		public int Depth
		{
			get
			{
				return depth;
			}
		}

		/// <summary>
		/// Returns the current token type.
		/// </summary>
		public RtfTokenType TokenType
		{
			get
			{
				return this.tokenType;
			}
		}

		/// <summary>
		/// Returns the value of the current token.
		/// </summary>
		public string Token
		{
			get
			{
				return this.token.ToString();
			}
		}

		/// <summary>
		/// Advances the reader to the next token in the stream.
		/// </summary>
		/// <returns>A boolean value indicating if there are more tokens in the stream.</returns>
		public bool Read()
		{
			if (this.atEndOfStream)
			{
				return false;
			}
			else
			{
				ReadNextToken();
				return true;
			}
		}

		#region Private Methods

		/// <summary>
		/// Reads the stream until a new token is found.
		/// </summary>
		private void ReadNextToken()
		{
			this.token = new StringBuilder();
			switch (currentChar)
			{
				case '{':
					this.depth++;
					this.tokenType = RtfTokenType.BlockOpener;
					ReadNextChar();
					break;

				case '}':
					this.depth--;
					this.tokenType = RtfTokenType.BlockCloser;
					ReadNextChar();
					return;

				case '\\':
					this.tokenType = RtfTokenType.Command;
					// Loop until the token is ready to be returned.
					do
					{
						// If after the backslash there is an apostrophe, this is an Escape token
						if (this.currentChar == '\'')
						{
							this.tokenType = RtfTokenType.Escape;
							for (int i = 0; i < 3; i++)
							{
								this.token.Append(currentChar);
								ReadNextChar();
							}
							break;
						}

						// If after the backslash there is a "~", this is a non-breaking space
						if (this.currentChar == '~')
						{
							this.tokenType = RtfTokenType.NonBreakingSpace;
							this.token.Append(currentChar);
							ReadNextChar();
							break;
						}

						// If after the backslash there is a "-", this is an hyphenation point
						if (this.currentChar == '-')
						{
							this.tokenType = RtfTokenType.HyphenationPoint;
							this.token.Append(currentChar);
							ReadNextChar();
							break;
						}

						// If after the backslash there is a "_", this is a non-breaking hyphen
						if (this.currentChar == '_')
						{
							this.tokenType = RtfTokenType.NonBreakingHyphen;
							this.token.Append(currentChar);
							ReadNextChar();
							break;
						}

						// If after the backslash there is a "*", this is a star-construct
						// and the following command is a non-standard one.
						if (this.currentChar == '*')
						{
							this.tokenType = RtfTokenType.StarConstruct;
							this.token.Append(currentChar);
							ReadNextChar();
							break;
						}
						this.token.Append(currentChar);
						ReadNextChar();
					}
					while (!IsTokenCloserChar());
					break;

				case '\r':
					goto case ' ';

				case '\n':
					goto case ' ';

				case ' ':
					this.tokenType = RtfTokenType.WhiteSpace;
					// There can be several whitespace characters together, such as
					// a space after a carriage return; this puts everything together
					do
					{
						this.token.Append(currentChar);
						ReadNextChar();
					}
					while (char.IsWhiteSpace(currentChar));
					break;

				default:
					this.tokenType = RtfTokenType.Text;
					// Loop until the token is ready to be returned.
					do
					{
						this.token.Append(currentChar);
						ReadNextChar();
					}
					while (!IsTokenCloserChar());
					break;
			}
		}

		/// <summary>
		/// Encapsulates the reading of the stream and sets variables used in other methods.
		/// </summary>
		private void ReadNextChar()
		{
			byte [] current = new byte[1];
			int status = this.stream.Read(current, 0, 1);
			this.atEndOfStream = (status == 0);
			currentChar = (char) current[0];
		}

		/// <summary>
		/// Indicates whether the current character in the stream is a "token closer", that is,
		/// one that makes the reader stop reading the stream and return control to the client
		/// application.
		/// </summary>
		/// <returns>A boolean value</returns>
		private bool IsTokenCloserChar()
		{
			return ((currentChar == '\\') || (currentChar == '\r') || (currentChar == '\n') 
				|| (currentChar == ' ') || (currentChar == '{') || (currentChar == '}'));
		}

		#endregion

		#region IDisposable Members

		/// <summary>
		/// Performs cleanup before garbage collection.
		/// </summary>
		public void Dispose()
		{
			this.stream.Close();
			this.stream = null;
		}

		#endregion
	}
}
