using System;
using System.Xml;
using System.Xml.XPath;

namespace Kosmaczewski.Oruga.XslFo
{
	/// <summary>
	/// Represents an XML document that conforms to the XSL-FO specification, as shown in
	/// http://www.w3.org/TR/xsl/
	/// </summary>
	public class XslFoDocument
	{
		private XmlDocument dom = new XmlDocument();

		/// <summary>
		/// Public constructor. Performs initialization of the instance.
		/// </summary>
		public XslFoDocument()
		{
			Init();
		}

		/// <summary>
		/// Creates an "fo:block" element representing a block of text in the document.
		/// </summary>
		/// <returns>A System.Xml.XmlElement instance representing a block</returns>
		public XmlElement CreateBlock()
		{
			return dom.CreateElement("fo:block", "http://www.w3.org/1999/XSL/Format");
		}

		/// <summary>
		/// Creates an "fo:inline" element representing an inline block of text in the document.
		/// </summary>
		/// <returns>A System.Xml.XmlElement instance representing an inline block</returns>
		public XmlElement CreateInline()
		{
			return dom.CreateElement("fo:inline", "http://www.w3.org/1999/XSL/Format");
		}

		/// <summary>
		/// Appends the fo:block instance to the current document.
		/// </summary>
		/// <param name="block">The block to append.</param>
		public void AppendBlock(XmlElement block)
		{
			XmlNamespaceManager nsmgr = new XmlNamespaceManager(this.dom.NameTable);
			nsmgr.AddNamespace("fo", "http://www.w3.org/1999/XSL/Format");
			dom.SelectSingleNode("fo:root/fo:page-sequence/fo:flow", nsmgr).AppendChild(block);
		}

		/// <summary>
		/// Creates an object to be used by XSL transformations by the ISerializers.
		/// <see cref="Kosmaczewski.Oruga.Html.HtmlSerializer.Write(System.IO.Stream)"/>
		/// </summary>
		/// <returns>An XPathNavigator instance.</returns>
		/// <remarks>The method of doing an XSL transformation using XPathNavigator
		/// objects is marked as "Obsolete" in the .NET Framework version 1.1.</remarks>
		public XPathNavigator GetNavigator()
		{
			return this.dom.CreateNavigator();
		}

		/// <summary>
		/// Saves the file to the specified filename.
		/// </summary>
		/// <param name="filename">Full path of the file that will be used to save the document.</param>
		public void Save(string filename)
		{
			this.dom.Save(filename);
		}

		/// <summary>
		/// Returns a string representation of the current instance.
		/// </summary>
		/// <returns>A string with the XML contents of the current instance.</returns>
		public override string ToString()
		{
			return dom.InnerXml;
		}

		#region Private Methods

		/// <summary>
		/// Performs the initialization of this instance.
		/// </summary>
		/// <remarks>
		/// This code initializes the dom to this XML:
		/// <code>
		/// &lt;?xml version="1.0" encoding="utf-8"?&gt;&lt;fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format"&gt;&lt;fo:layout-master-set&gt;&lt;fo:simple-page-master master-name="my-page"&gt;&lt;fo:region-body margin="1in" /&gt;&lt;/fo:simple-page-master&gt;&lt;/fo:layout-master-set&gt;&lt;fo:page-sequence master-reference="my-page"&gt;&lt;fo:flow flow-name="xsl-region-body" font="12pt Times"/&gt;&lt;/fo:page-sequence&gt;&lt;/fo:root&gt;
		/// </code>
		/// Following XSL-FO specs, "fo:block" elements are to be appended to the "fo:flow" element.
		/// </remarks>
		private void Init()
		{
			/*
			 * This code initializes the dom to this XML:
			 * 
			 * <?xml version="1.0" encoding="utf-8"?>
			 * <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
			 *		<fo:layout-master-set>
			 *			<fo:simple-page-master master-name="my-page">
			 *				<fo:region-body margin="1in" />
			 *			</fo:simple-page-master>
			 *		</fo:layout-master-set>
			 *		<fo:page-sequence master-reference="my-page">
			 *			<fo:flow flow-name="xsl-region-body" font="12pt Times"/>
			 *		</fo:page-sequence>
			 * </fo:root>
			 * 
			 * Following XSL-FO specs, "fo:block" elements are to be appended to the "fo:flow" element.
			 * 
			 * */
			XmlAttribute margin = dom.CreateAttribute("margin");
			margin.Value = "1in";

			XmlAttribute masterName = dom.CreateAttribute("master-name");
			masterName.Value = "my-page";

			XmlElement regionBody = dom.CreateElement("fo:region-body", "http://www.w3.org/1999/XSL/Format");
			regionBody.Attributes.Append(margin);

			XmlElement simplePageMaster = dom.CreateElement("fo:simple-page-master", "http://www.w3.org/1999/XSL/Format");
			simplePageMaster.Attributes.Append(masterName);
			simplePageMaster.AppendChild(regionBody);

			XmlElement layoutMasterSet = dom.CreateElement("fo:layout-master-set", "http://www.w3.org/1999/XSL/Format");
			layoutMasterSet.AppendChild(simplePageMaster);

			XmlDeclaration declaration = dom.CreateXmlDeclaration("1.0", "utf-8", null);
			dom.AppendChild(declaration);

			XmlAttribute flowName = dom.CreateAttribute("flow-name");
			flowName.Value = "xsl-region-body";

			XmlAttribute font = dom.CreateAttribute("font");
			font.Value = "12pt Times";

			XmlAttribute masterReference = dom.CreateAttribute("master-reference");
			masterReference.Value = "my-page";

			XmlElement flow = dom.CreateElement("fo:flow", "http://www.w3.org/1999/XSL/Format");
			flow.Attributes.Append(flowName);
			flow.Attributes.Append(font);

			XmlElement pageSequence = dom.CreateElement("fo:page-sequence", "http://www.w3.org/1999/XSL/Format");
			pageSequence.Attributes.Append(masterReference);
			pageSequence.AppendChild(flow);

			XmlElement root = dom.CreateElement("fo:root", "http://www.w3.org/1999/XSL/Format");
			root.AppendChild(layoutMasterSet);
			root.AppendChild(pageSequence);
			dom.AppendChild(root);
		}

		#endregion
	}
}
