#!/usr/bin/perl
require 5;
use strict;
use warnings;
# (Run under a sane version of Perl, with optimal error-checking)

#--------------------------------------------------------------------------
# Declare some constants:

use constant A4 => 0;  # Change 0 to 1 if printing on A4 paper

use constant Inches2twips => 1440; # (conversion factor)

use constant Page_Width  =>  A4 ? 11909 : (8.5 * Inches2twips);
use constant Page_Height =>  A4 ? 16834 : ( 11 * Inches2twips);

use constant Smidgen     => 1/8 * Inches2twips; # fudge factor
use constant CD_Diameter => Smidgen + 4.75 * Inches2twips;
use constant Wing_Size   => (Page_Width - CD_Diameter) / 2;

use constant First_Line_Height =>  40 * 10;

#--------------------------------------------------------------------------
# Declare the two global variables:
my $out = "cd_case.rtf";
my @lines;

# The main work of the program:

get_input();
open(RTF, ">$out") or die "Can't write-open $out: $!";
doc_intro();
draw_fold_guides();
print_text();
close(RTF);
print "\nCreated $out (", -s $out, " bytes)\n";
exit;

#--------------------------------------------------------------------------

sub get_input {
  print "Enter text for CD case, then hit Enter and EOF (", 
    ($^O =~ m/MSWin/) ? "control-z)\n" : "control-d)\n"
   unless @ARGV;
    # Which key you hit depends on your OS.  MSWin uses
    #  control-z, whereas most everything else by default
    #  uses control-d.
  
  @lines = <>;  # Get the input lines (either from a file,
   # or from standard input / the console)
  
  unless( grep m/\S/, @lines) { # if all blank
    print "\nNo input lines?  Making a blank case then.\n";
    @lines = ( '_' ); # just a dummy line
  }
  
  chomp(@lines); # remove the \n's
  return;  
}

#--------------------------------------------------------------------------

sub doc_intro { # Start off the RTF file

  printf RTF '{\rtf1\ansi
\deff0{\fonttbl {\f0 \froman Times New Roman;}}
\paperw%s \paperh%s \deflang1033\plain\f0\fs20

{\pard\par}

', int(.5 + Page_Width), int(.5 + Page_Height);

  return;
}

#--------------------------------------------------------------------------

sub print_text {
  # Write the contents of @lines into this exact-positioned
  # paragraph, feeding each one through esc($line).

  printf RTF '{\pard \pvpg\phpg
\posx%s \posy%s
\absw%s \absh-%s
',  map int($_ + .5), # make everything an integer

    Wing_Size + Smidgen, 
    Page_Height - Page_Height * 1/4
     - CD_Diameter - First_Line_Height,
    CD_Diameter - 2 * Smidgen,
    CD_Diameter - 2 * Smidgen + First_Line_Height,
  ;

  print RTF "\n\\f0\\fs20{\\fs40 ",
    # Make the first line larger
    @lines ? esc(shift @lines) : '',
    "}\n";

  foreach my $line (@lines) {
    print RTF "\n\\line ", esc($line), "\n";
  }

  print RTF "\n\\par}\n}"; # end box and document
  return;
}

#--------------------------------------------------------------------------

sub esc {   # return the given string, escaped as good RTF
  my $in = $_[0];
  $in =~ s{([^-"'?,\$\._a-zA-Z0-9 ])}
          {sprintf("\\'%02x",ord($1))}eg;
   # actually escapes much more than is necessary
  return $in;
}

#--------------------------------------------------------------------------

sub draw_fold_guides { # draw all the lines!

  # draw ------- across bottom
  line(0,           Page_Height*3/4,
       Page_Width,  Page_Height*3/4 );
  
  # draw  |  |   from top to bottom
  line(Wing_Size + Smidgen/2, 0,
       Wing_Size, Page_Height );
  line(Page_Width - Wing_Size - Smidgen/2, 0,
       Page_Width - Wing_Size,  Page_Height );
  
  # draw \    /  by the bottom line
  #      /    \
  line( Wing_Size, Page_Height*3/4,           
        0, (Page_Height*3/4) + Wing_Size );
  line( Wing_Size, Page_Height*3/4,           
        0, (Page_Height*3/4) - Wing_Size );
  line( Page_Width - Wing_Size, Page_Height*3/4,
        Page_Width, (Page_Height*3/4) + Wing_Size );
  line( Page_Width - Wing_Size, Page_Height*3/4,
        Page_Width, (Page_Height*3/4) - Wing_Size );
  return;
}

#--------------------------------------------------------------------------
# And finally, the routine that handles the grunt-work
# of actually generating the RTF code for a line from
# one page-point to another.

my $z_count;  # just a counter variable

sub line {  # make a line from (x,y) to (x2,y2)
  my($x1,$y1,$x2,$y2) = map int(.5 + $_), @_;

  # Make sure we have four arguments
  use Carp;
  croak "line() takes four parameters" unless @_ == 4;
  
  # Get the page coordinates of northwest corner
  my $nw_x = ($x1 < $x2) ? $x1 : $x2;
  my $nw_y = ($y1 < $y2) ? $y1 : $y2;
  
  $x1 -= $nw_x; $x2 -= $nw_x; $y1 -= $nw_y; $y2 -= $nw_y;
  # So that (x1,y1) and (x2,y2) are relative to the NW corner

  my $w = abs($x1 - $x2); # horizontal distance
  my $h = abs($y1 - $y2); # vertical distance
  $z_count++;
  
  printf RTF '{\*\do\dobxpage\dobypage\dodhgt%s\dpline
\dpptx%s\dppty%s \dpptx%s\dppty%s
\dpx%s\dpy%s \dpxsize%s\dpysize%s
\dplinew%s \dplinecor%s\dplinecog%s\dplinecob%s
}

',  $z_count,
    $x1,$y1, $x2,$y2,
    $nw_x, $nw_y, $w, $h,
    15,  # line thickness
    0,0,0, # RGB of line color
  ;
  return;
}

__END__


=head2 An Origami CD Case Maker

This program generates a single page that, when folded correctly, makes
a CD case.  Such cases are quite useful for all the CDRs that always
manage to outnumber available plastic CD cases.
[

Footnote:

Many thanks for the original concept go to Tom Hull from the Merrimack
University mathematics department.  There's more at
http://web.merrimack.edu/~thull/ and
http://papercdcase.com/

]
This program works by getting the lines of text that you want to print
on the label, and then generating an RTF file with that text placed
exactly on the page, along with lines that mark where to fold.  (The
first few times, you'll need to refer to the illustration here to
see which direction to fold in, and in what order.)

[...]

The program can take input from a file, as with:

  % cdcase title_and_tracklist.txt

Or it can take input from standard input, i.e.:

  % cdcase < title_and_tracklist.txt

Or:

  % ls -l ./backups5 | cdcase

Or it can just read input from the console:

  % cdcase
  Enter text for CD case, then hit Enter and EOF (control-z)
  The CD Title
  Title for track 1
  Title for track 2
  Title for track 3
  [then hit control-D or control-Z on a new line]

In any case, unless you hit break (typically control-c), the program
will write the CD case to the file F<cd_case.rtf>. If you provide no
input, it will write a blank case -- that is, a page with no text, but
with all the fold lines. You can open that file in a word processor and
edit it to have whatever text you like, before printing it, folding
it up, and using it.

=cut



